package mvd.jester.simulator;

import java.util.HashSet;
import java.util.Set;
import com.google.common.math.LongMath;
import mvd.jester.model.SystemSetup;
import mvd.jester.model.Task;
import mvd.jester.priority.RateMonotonic;
import mvd.jester.simulator.internals.ProcessorContext;
import mvd.jester.simulator.internals.SortedTaskContextSet;
import mvd.jester.simulator.internals.TaskContext;


/**
 * AbstractSimulator
 */
public abstract class AbstractSimulator implements SimulatorInterface {

    protected final SystemSetup systemSetup;
    protected final Set<ProcessorContext> processors;
    protected final SortedTaskContextSet readyTasks;
    protected final long hyperPeriod;

    AbstractSimulator(SystemSetup systemSetup) {
        this.systemSetup = systemSetup;
        this.readyTasks = new SortedTaskContextSet(new RateMonotonic());
        processors = new HashSet<>();
        for (int i = 0; i < systemSetup.getNumberOfProcessors(); ++i) {
            processors.add(new ProcessorContext(i));
        }
        this.hyperPeriod = systemSetup.getTasks().last().getPeriod() * 10;
        // LongMath.pow(systemSetup.getTasks().last().getPeriod(), 2);
    }


    protected boolean releaseTasks(long timeStep) {
        for (Task t : systemSetup.getTasks()) {
            if (timeStep % t.getPeriod() == 0) {
                if (!readyTasks.add(new TaskContext(t, timeStep))) {
                    return false;
                }
            }
        }
        return true;
    }

}
