package mvd.jester.simulator;

import java.util.Comparator;
import java.util.Optional;
import java.util.TreeSet;
import mvd.jester.model.SystemSetup;
import mvd.jester.simulator.internals.ProcessorContext;
import mvd.jester.simulator.internals.TaskContext;

/**
 * MaiaBertogna
 */
public class MaiaBertogna extends AbstractSimulator {

    public MaiaBertogna(SystemSetup systemSetup) {
        super(systemSetup);
    }

    @Override
    public boolean runSimulation() {
        readyTasks.clear();
        for (int t = 0; t < hyperPeriod; ++t) {
            if (!releaseTasks(t)) {
                return false;
            }

            TreeSet<ProcessorContext> sortedProcessors =
                    new TreeSet<>((p1, p2) -> (int) (!p1.getJob().isPresent() ? -1
                            : !p2.getJob().isPresent() ? 1 : -1));

            processors.forEach(p -> sortedProcessors.add(p));
            for (ProcessorContext p : sortedProcessors) {
                for (TaskContext tc : readyTasks) {
                    if (p.acceptTask(tc, t)) {
                        break;
                    }
                }

            }

            for (ProcessorContext p : processors) {
                Optional<TaskContext> optionalTc = p.updateExecution(t);
                if (optionalTc.isPresent()) {
                    TaskContext tc = optionalTc.get();
                    if (t >= tc.getDeadline()) {
                        return false;
                    }
                    readyTasks.remove(optionalTc.get());
                }
            }
        }

        return true;
    }

    @Override
    public String getName() {
        return "MaiaBertogna";
    }


}
